﻿using System;
using System.Windows.Forms;

namespace DelegateExamplesConsole
{
    delegate void DelegateWithParam(String message);
    delegate void DelegateNoParam();

    public class Program
    {
        public static void Main(string[] args)
        {
            if (args.Length > 0)
            {
                switch (args[0].ToLower())
                {
                    case "lambdat":
                        LambdaWithParam(args);
                        break;
                    case "lambda":
                        LambdaNoParam(args);
                        break;
                    case "anont":
                        AnonymousWithParam(args);
                        break;
                    case "anon":
                        AnonymousNoParam(args);
                        break;
                    case "actiont":
                        ActionWithParam(args);
                        break;
                    case "action":
                        ActionNoParam(args);
                        break;
                    case "delegatet":
                        DelegateWithParam(args);
                        break;
                    default:
                        DelegateNoParam(args);
                        break;

                }
            } else
            {
                Console.WriteLine("Enter delegate, delegateT, action, actionT, anon, anonT, lambda, lambdaT");
            }
            Console.Write("Press Enter to close...");
            Console.ReadLine();

        }

        private static void DelegateNoParam(String[] args)
        {
            DelegateNoParam messageTarget;
            NameClass name = new NameClass("Hello, Custom Delegate()!");
            if (args.Length > 1)
            {
                messageTarget = name.DislpayToWindow;
            }
            else
            {
                messageTarget = name.DisplayToConsole;
            }
            messageTarget();
        }

        private static void DelegateWithParam(String[] args)
        {
            DelegateWithParam messageTarget;
            if (args.Length > 1)
            {
                messageTarget = ShowWindowMessage;
            }
            else
            {
                messageTarget = Console.WriteLine;
            }
            messageTarget("Hello, Custom Delegate(string)!");

        }

        private static void ActionNoParam(String[] args)
        {
            Action messageTarget;
            NameClass name = new NameClass("Hello, Action!");
            if (args.Length > 1)
            {
                messageTarget = name.DislpayToWindow;
            }
            else
            {
                messageTarget = name.DisplayToConsole;
            }
            messageTarget();
        }

        private static void ActionWithParam(String[] args)
        {
            Action<String> messageTarget;
            if (args.Length > 1)
            {
                messageTarget = ShowWindowMessage;
            }
            else
            {
                messageTarget = Console.WriteLine;
            }
            messageTarget("Hello, Action<String>!");

        }

        private static void AnonymousNoParam(String[] args)
        {
            Action messageTarget;
            NameClass name = new NameClass("Hello, anonymous!");
            if (args.Length > 1)
            {
                messageTarget = delegate () { name.DislpayToWindow(); };
            }
            else
            {
                messageTarget = delegate () { name.DisplayToConsole(); };
            }
            messageTarget();
        }

        private static void AnonymousWithParam(String[] args)
        {
            Action<String> messageTarget;
            if (args.Length > 1)
            {
                messageTarget = delegate (string s) { ShowWindowMessage(s); };
            }
            else
            {
                messageTarget = delegate (string s) { Console.WriteLine(s); };
            }
            messageTarget("Hello, anonymous(string)!");

        }

        private static void LambdaNoParam(String[] args)
        {
            Action messageTarget;
            NameClass name = new NameClass("Hello, lambda!");
            if (args.Length > 1)
            {
                messageTarget = () => { name.DislpayToWindow(); };
            }
            else
            {
                messageTarget = () => { name.DisplayToConsole(); };
            }
            messageTarget();
        }

        private static void LambdaWithParam(String[] args)
        {
            Action<String> messageTarget;
            if (args.Length > 1)
            {
                messageTarget = (s) => { ShowWindowMessage(s); };
            }
            else
            {
                messageTarget = (s) => { Console.WriteLine(s); };
            }
            messageTarget("Hello, lambda(string)!");

        }

        private static void ShowWindowMessage(String message)
        {
            MessageBox.Show(message);
        }
    }
}
